@@ -25,10 +25,10 @@
    Pavel Cisler <pavel@eazel.com> 
 */
 
-/* FIXME bugzilla.eazel.com 1197: 
+/* FIXME bugzilla.gnome.org #41197: 
    Check that all the flags passed by address are set at least once by
    functions that are expected to set them.  */
-/* FIXME bugzilla.eazel.com 1198: 
+/* FIXME bugzilla.gnome.org #41198: 
    There should be a concept of a `context' in the file methods that would
    allow us to set a prefix URI for all the operations.  This way we can
    greatly optimize access to "strange" file systems.  */
@@ -113,6 +113,8 @@ free_progress (GnomeVFSXferProgressInfo *progress_info)
 	progress_info->source_name = NULL;
 	g_free (progress_info->target_name);
 	progress_info->target_name = NULL;
+	g_free (progress_info->duplicate_name);
+	progress_info->duplicate_name = NULL;
 }
 
 static void
@@ -1234,7 +1236,6 @@ copy_file (GnomeVFSFileInfo *info,
 {
 	GnomeVFSResult close_result, result;
 	GnomeVFSHandle *source_handle, *target_handle;
-	GnomeVFSSetFileInfoMask set_mask;
 
 	progress->progress_info->phase = GNOME_VFS_XFER_PHASE_OPENSOURCE;
 	progress->progress_info->bytes_copied = 0;
@@ -1296,21 +1297,21 @@ copy_file (GnomeVFSFileInfo *info,
 	}
 
 	if (result == GNOME_VFS_OK) {
+		/* ignore errors while setting file info attributes at this point */
+		
 		/* FIXME the modules should ignore setting of permissions if
 		 * "valid_fields & GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS" is clear
 		 * for now, make sure permissions aren't set to 000
 		 */
-
-		if ((info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS) == 0) {
-			set_mask = GNOME_VFS_SET_FILE_INFO_TIME;
-		} else {
-			set_mask = GNOME_VFS_SET_FILE_INFO_PERMISSIONS
-					| GNOME_VFS_SET_FILE_INFO_OWNER
-					| GNOME_VFS_SET_FILE_INFO_TIME;
+		if ((info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS) != 0) {
+			/* Call this separately from the time one, since one of them may fail,
+			   making the other not run. */
+			gnome_vfs_set_file_info_uri (target_uri, info, 
+						     GNOME_VFS_SET_FILE_INFO_OWNER | GNOME_VFS_SET_FILE_INFO_PERMISSIONS);
 		}
-
-		/* ignore errors while setting file info attributes at this point */
-		gnome_vfs_set_file_info_uri (target_uri, info, set_mask);
+		
+		/* Call this last so nothing else changes the times */
+		gnome_vfs_set_file_info_uri (target_uri, info, GNOME_VFS_SET_FILE_INFO_TIME);
 	}
 
 	if (*skip) {
@@ -1596,7 +1597,6 @@ copy_items (const GList *source_uri_list,
 		}
 
 		gnome_vfs_file_info_unref (info);
-		g_free (progress->progress_info->duplicate_name);
 
 		if (result != GNOME_VFS_OK) {
 			break;
@@ -2006,7 +2006,6 @@ gnome_vfs_new_directory_with_unique_name (const GnomeVFSURI *target_dir_uri,
 	}
 
 	gnome_vfs_uri_unref (target_uri);
-	g_free (progress->progress_info->duplicate_name);
 
 	return result;
 }
@@ -2144,7 +2143,7 @@ gnome_vfs_xfer_uri_internal (const GList *source_uris,
 
 
 		if (result == GNOME_VFS_OK) {
-			if (!move && progress->progress_info->bytes_total > free_bytes) {
+			if (!move && !link && progress->progress_info->bytes_total > free_bytes) {
 				result = GNOME_VFS_ERROR_NO_SPACE;
 				progress_set_source_target_uris (progress, NULL, target_dir_uri);
 				handle_error (&result, progress, &error_mode, &skip);
